# Copyright IBM Corp. 2017 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging

import rx

from hfc.fabric.channel.transactionproposals \
    import TransactionProposalHandler, TransactionProposalRequest, \
    CC_INSTANTIATE, check_tran_prop_request, \
    build_header, build_proposal, sign_proposal
from hfc.protos.common import common_pb2
from hfc.protos.peer import chaincode_pb2
from hfc.util.crypto import crypto
from hfc.util.utils import proto_str, proto_b

# deprecated remain for new code

_logger = logging.getLogger(__name__ + ".instantiate")


class Instantiation(TransactionProposalHandler):
    """Chaincode instantiate transaction proposal handler. """

    def handle(self, tran_prop_req, scheduler=None):
        """Execute chaincode instantiation transaction proposal request.

        :param tran_prop_req: chaincode instantiation transaction proposal request
        :param scheduler: see rx.Scheduler, defaults to None
        :return: An rx.Observer wrapper of chaincode instantiation response
        """
        return _instantiate_chaincode(self._chain, tran_prop_req, scheduler)


def _create_instantiation_proposal(tran_prop_req, chain):
    """Create a chaincode instantiation proposal
    This involves assembling the proposal with the data (chaincodeID,
    chaincode invocation spec, etc.) and signing it using the private key
    corresponding to the ECert to sign.

    :param tran_prop_req: see TransactionProposalRequest
    :param chain: chain instance
    :return: (Proposal): The created Proposal instance or None.
    """
    args = ["init" if not tran_prop_req.fcn
            else tran_prop_req.fcn] + tran_prop_req.args

    cc_deployment_spec = chaincode_pb2.ChaincodeDeploymentSpec()
    cc_deployment_spec.chaincode_spec.type = \
        chaincode_pb2.ChaincodeSpec.Type.Value('GOLANG')
    cc_deployment_spec.chaincode_spec.chaincode_id.name = \
        proto_str(tran_prop_req.chaincode_id)
    cc_deployment_spec.chaincode_spec.chaincode_id.path = \
        proto_str(tran_prop_req.chaincode_path)
    cc_deployment_spec.chaincode_spec.chaincode_id.version = \
        proto_str(tran_prop_req.chaincode_version)
    cc_deployment_spec.chaincode_spec.input.args.extend(list(map(
        lambda x: proto_b(x), args)))

    header = build_header(tran_prop_req.signing_identity,
                          tran_prop_req.nonce,
                          common_pb2.ENDORSER_TRANSACTION,
                          chain,
                          tran_prop_req.prop_type,
                          chaincode_id=tran_prop_req.chaincode_id
                          )

    cci_spec = chaincode_pb2.ChaincodeInvocationSpec()
    cci_spec.chaincode_spec.type = \
        chaincode_pb2.ChaincodeSpec.Type.Value('GOLANG')
    cci_spec.chaincode_spec.chaincode_id.name = proto_str("lscc")
    cci_spec.chaincode_spec.input.args.extend(
        [proto_b(CC_INSTANTIATE), proto_b('default'),
         cc_deployment_spec.SerializeToString()])
    proposal = build_proposal(cci_spec, header)

    signed_proposal = sign_proposal(
        tran_prop_req.signing_identity, proposal)
    return signed_proposal


def _instantiate_chaincode(chain, cc_instantiation_request, scheduler=None):
    """Instantiate chaincode.

    :param chain: chain instance
    :param cc_instantiation_request: see TransactionProposalRequest
    :param scheduler: see rx.Scheduler, defaults to None
    :return: An rx.Observable of instantiation response
    """
    if len(chain.peers) < 1:
        return rx.Observable.just(ValueError(
            "Missing peer objects on this chain"
        ))

    peers = {}
    if cc_instantiation_request and cc_instantiation_request.targets:
        peers = cc_instantiation_request.targets
        for peer in peers:
            if not chain.is_valid_peer(peer):
                return rx.Observable.just(ValueError(
                    'Request targets peer object {} not in chain'.format(peer)
                ))

    if len(peers) < 1:
        peers = chain.peers

    return rx.Observable \
        .just(cc_instantiation_request) \
        .map(check_tran_prop_request) \
        .map(lambda req, idx: _create_instantiation_proposal(req, chain))
    # .flatmap(lambda proposal, idx:
    #          send_transaction_proposal(proposal, peers, scheduler))


def create_instantiation_proposal_req(chaincode_id, chaincode_path,
                                      chaincode_version, creator,
                                      fcn='init', args=None,
                                      nonce=crypto.generate_nonce(24),
                                      targets=None):
    """Create instantiation proposal request.

    :param chaincode_id: chaincode_id
    :param chaincode_path: chaincode_path
    :type chaincode_path: str
    :param chaincode_version: chaincode_version
    :type chaincode_version: str
    :param creator: user
    :param fcn: chaincode init function, defaults to 'init'
    :param args: init function args, defaults to None
    :param nonce: nonce, defaults to crypto.generate_nonce(24)
    :param targets: peers, defaults to None
    :return: see TransactionProposalRequest
    """
    return TransactionProposalRequest(chaincode_id, creator, CC_INSTANTIATE,
                                      chaincode_path, chaincode_version,
                                      fcn=fcn, args=args,
                                      nonce=nonce, targets=targets)


def chaincode_instantiation(chain):
    """Create instantiate.

    :param chain: chain instance
    :return: Instantiate instance
    """
    return Instantiation(chain)



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import unittest
from hfc.fabric.client import Client


loop = asyncio.get_event_loop()


class ClientTest(unittest.TestCase):

    def setUp(self):
        self.client = Client('test/fixtures/network.json')

    def test_create_client(self):

        self.client.crypto_suite = 'my_crypto_suite'
        self.assertEqual(self.client.crypto_suite, 'my_crypto_suite')

        self.client.tx_context = 'my_tx_context'
        self.assertEqual(self.client.tx_context, 'my_tx_context')

        self.client.user_context = 'my_user_context'
        self.assertEqual(self.client.user_context, 'my_user_context')

        self.client.state_store = 'my_state_store'
        self.assertEqual(self.client.state_store, 'my_state_store')

    def test_new_channel(self):
        test_channel = self.client.new_channel('test')
        self.assertEqual(test_channel, self.client.get_channel('test'))

    def test_get_channel(self):
        test_channel = self.client.new_channel('test')
        self.assertEqual(test_channel, self.client.get_channel('test'))

        no_chain = self.client.get_channel('test1')
        self.assertIsNone(no_chain)

    def test_create_channel_missing_signatures(self):
        request = {}
        request['config'] = 'config'
        request['channel_name'] = 'channel_name'
        request['orderer'] = 'orderer'
        request['tx_id'] = 'tx_id'
        request['nonce'] = 'nonce'
        with self.assertRaises(ValueError):
            loop.run_until_complete(self.client._create_or_update_channel(
                request))

    def test_create_channel_not_list_of_signatures(self):
        request = {}
        request['config'] = 'config'
        request['signatures'] = 'signatures'
        request['channel_name'] = 'channel_name'
        request['orderer'] = 'orderer'
        request['tx_id'] = 'tx_id'
        request['nonce'] = 'nonce'
        with self.assertRaises(ValueError):
            loop.run_until_complete(self.client._create_or_update_channel(
                request))

    def test_create_channel_missing_tx_id(self):
        request = {}
        request['config'] = 'config'
        request['channel_name'] = 'channel_name'
        request['orderer'] = 'orderer'
        request['nonce'] = 'nonce'

        with self.assertRaises(ValueError):
            loop.run_until_complete(self.client._create_or_update_channel(
                request))

    def test_create_channel_missing_orderer(self):
        request = {}
        request['config'] = 'config'
        request['channel_name'] = 'channel_name'
        request['tx_id'] = 'tx_id'
        request['nonce'] = 'nonce'

        with self.assertRaises(ValueError):
            loop.run_until_complete(self.client._create_or_update_channel(
                request))

    def test_create_channel_missing_channel_name(self):
        request = {
            'config': 'config',
            'orderer': 'orderer',
            'tx_id': 'tx_id',
            'nonce': 'nonce',
        }

        with self.assertRaises(ValueError):
            loop.run_until_complete(self.client._create_or_update_channel(
                request))

    def test_export_network_profile(self):
        network_info = {
            "organizations": {
            },
            "orderers": {
            },
            "peers": {
            },
            "certificateAuthorities": {
            },
            'a': {
                'b': 'a.b'
            }
        }
        self.client.network_info = network_info
        self.client.export_net_profile('test-export.json')
        self.client.network_info = dict()
        self.client.init_with_net_profile('test-export.json')
        self.assertEqual(network_info, self.client.network_info)
        self.assertEqual('a.b', self.client.get_net_info('a', 'b'))

    def test_init_with_net_profile(self):
        self.client.init_with_net_profile('test/fixtures/network.json')
        self.assertEqual(self.client.get_net_info('organizations',
                                                  'orderer.example.com',
                                                  'mspid'), 'OrdererMSP',
                         "profile not match")


if __name__ == '__main__':
    unittest.main()



# Copyright IBM Corp. 2016 All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import base64
import json
import logging

from numbers import Number

import requests
import six
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import Encoding
from cryptography.x509 import NameOID

from hfc.fabric_ca.certificateService import CertificateService
from hfc.fabric_ca.identityService import IdentityService
from hfc.fabric_ca.affiliationService import AffiliationService
from hfc.util.crypto.crypto import ecies

DEFAULT_CA_ENDPOINT = 'http://localhost:7054'
DEFAULT_CA_BASE_URL = '/api/v1/'

_logger = logging.getLogger(__name__)

reasons = (
    (1, 'unspecified'),
    (2, 'keycompromise'),
    (3, 'cacompromise'),
    (4, 'affiliationchange'),
    (5, 'superseded'),
    (6, 'cessationofoperation'),
    (7, 'certificatehold'),
    (8, 'removefromcrl'),
    (9, 'privilegewithdrawn'),
    (10, 'aacompromise')
)


class Enrollment(object):
    """Class represents enrollment."""

    def __init__(self, private_key, enrollmentCert, caCertChain=None,
                 service=None):
        self._service = service
        self._private_key = private_key
        self._cert = enrollmentCert
        self._caCert = caCertChain

    @property
    def private_key(self):
        """Get private key

        :return: private key
        """
        return self._private_key

    @private_key.setter
    def private_key(self, private_key):
        """Set the private key

        :param private_key: private key
        :return:
        """
        self._private_key = private_key

    @property
    def cert(self):
        """Get cert

        :return: cert
        """
        return self._cert

    @cert.setter
    def cert(self, cert):
        """Set cert

        :param cert: cert
        :return:
        """
        self._cert = cert

    @property
    def caCert(self):
        """Get caCert

        :return: caCert
        """
        return self._caCert

    @caCert.setter
    def caCert(self, caCert):
        """Set caCert

        :param caCert: caCert
        :return:
        """
        self._caCert = caCert

    def get_attrs(self):
        return ",".join("{}={}"
                        .format(k, getattr(self, k))
                        for k in self.__dict__.keys())

    def register(self, enrollmentID, enrollmentSecret=None, role=None,
                 affiliation=None, maxEnrollments=1, attrs=None):

        # TODO, default should be equal to registrar
        # https://hyperledger-fabric-ca.readthedocs.io/en/latest/users-guide.ht
        # ml#registering-a-new-identity
        if affiliation is None:
            affiliation = ''

        if not enrollmentID:
            msg = 'Missing required parameter. \'enrollmentID\' is required.'
            raise ValueError(msg)
        if not isinstance(maxEnrollments, Number):
            msg = 'Wrong parameter. \'maxEnrollments\' must be a' \
                  ' number'
            raise ValueError(msg)

        return self._service.register(enrollmentID, enrollmentSecret, role,
                                      affiliation, maxEnrollments, attrs, self)

    # https://hyperledger-fabric-ca.readthedocs.io/en/latest/users-guide.html
    # #revoking-a-certificate-or-identity
    def revoke(self, enrollmentID=None, aki=None, serial=None,
               reason=None, gencrl=False):
        if not enrollmentID:
            if not aki or not serial:
                msg = 'Enrollment ID is empty, thus both "aki" and "serial"' \
                      ' must have non-empty values'
                raise ValueError(msg)

        if reason and reason not in [r[1] for r in reasons]:
            msg = 'Reason is not a valid one. Please review reasons listed' \
                  ' in fabric-ca specifications.'
            raise ValueError(msg)

        return self._service.revoke(enrollmentID, aki, serial,
                                    reason, gencrl, self)

    def generateCRL(self, revokedBefore=None, revokedAfter=None,
                    expireBefore=None, expireAfter=None):

        if revokedAfter and revokedBefore:
            try:
                if revokedAfter > revokedBefore:
                    msg = 'revokedAfter Date cannot be greater than the' \
                          ' revokedBefore Date'
                    raise ValueError(msg)
            except Exception:
                msg = 'revokedAfter and revokedBefore are not Date'
                raise ValueError(msg)

        # pass date to YYYY-MM-DDTHH:mm:ss.sssZ
        try:
            revokedBefore = revokedBefore.isoformat()
        except Exception:
            revokedBefore = None

        try:
            revokedAfter = revokedAfter.isoformat()
        except Exception:
            revokedAfter = None

        try:
            expireBefore = expireBefore.isoformat()
        except Exception:
            expireBefore = None

        try:
            expireAfter = expireAfter.isoformat()
        except Exception:
            expireAfter = None

        return self._service.generateCRL(revokedBefore, revokedAfter,
                                         expireBefore, expireAfter, self)

    def __str__(self):
        return "[{}:{}]".format(self.__class__.__name__, self.get_attrs())


class CAClient(object):
    """Client for communicating with the Fabric CA APIs."""

    def __init__(self, target=DEFAULT_CA_ENDPOINT, ca_certs_path=None,
                 base_url=DEFAULT_CA_BASE_URL, ca_name="",
                 cryptoPrimitives=ecies()):
        """ Init CA client.

        :param target: CA server address including protocol,hostname,port
        :param ca_certs_path: Local ca certs path
        :param ca_name: The optional name of the CA. Fabric-ca servers
        support multiple Certificate Authorities from a single server
        If omitted or null or an empty string, then the default CA is
        the target of requests.
        :return: An instance of CAClient
        """
        self._ca_certs_path = ca_certs_path
        self._base_url = target + base_url
        self._ca_name = ca_name
        self._cryptoPrimitives = cryptoPrimitives

    def generateAuthToken(self, req, registrar):
        """Generate authorization token required for accessing fabric-ca APIs

        :param req: request body
        :type req: dict
        :param registrar: Required. The identity of the registrar
        (i.e. who is performing the request)
        :type registrar: Enrollment
        :return: auth token
        """
        b64Cert = base64.b64encode(registrar._cert)

        if req:
            reqJson = json.dumps(req, ensure_ascii=False)
            b64Body = base64.b64encode(reqJson.encode())

            # /!\ cannot mix f format and b
            # https://stackoverflow.com/questions/45360480/is-there-a-
            # formatted-byte-string-literal-in-python-3-6
            bodyAndCert = b'%s.%s' % (b64Body, b64Cert)
        else:
            bodyAndCert = b'.%s' % b64Cert

        sig = self._cryptoPrimitives.sign(registrar._private_key, bodyAndCert)
        b64Sign = base64.b64encode(sig)

        # /!\ cannot mix f format and b
        return b'%s.%s' % (b64Cert, b64Sign)

    def _send_ca_post(self, path, **param):
        """Send a post request to the ca service

        :param path: sub path after the base_url
        :param **param: post request params
        :return: the response body in json
        """
        r = requests.post(url=self._base_url + path, **param)
        return r.json(), r.status_code

    def _send_ca_get(self, path, **param):
        """Send a get request to the ca service

        :param path: sub path after the base_url
        :param **param: get request params
        :return: the response body in json
        """
        r = requests.get(url=self._base_url + path, **param)
        return r.json(), r.status_code

    def _send_ca_delete(self, path, **param):
        """Send a delete request to the ca service

        :param path: sub path after the base_url
        :param **param: delete request params
        :return: the response body in json
        """
        r = requests.delete(url=self._base_url + path, **param)
        return r.json(), r.status_code

    def _send_ca_update(self, path, **param):
        """Send a update request to the ca service

        :param path: sub path after the base_url
        :param **param: update request params
        :return: the response body in json
        """
        r = requests.put(url=self._base_url + path, **param)
        return r.json(), r.status_code

    def get_cainfo(self):
        """Query the ca service information.

        :return: The base64 encoded CA PEM file content for the caname
        """
        if self._ca_name != "":
            body_data = {"caname": self._ca_name}
        else:
            body_data = {}
        res, st = self._send_ca_post(path="cainfo", json=body_data,
                                     verify=self._ca_certs_path)
        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        if res['success'] and res['result']['CAName'] == self._ca_name:
            return base64.b64decode(res['result']['CAChain'])
        else:
            raise ValueError("get_cainfo failed with errors {0}"
                             .format(res['errors']))

    def enroll(self, enrollment_id, enrollment_secret, csr, profile='',
               attr_reqs=None):
        if not enrollment_id or not enrollment_secret or not csr:
            raise ValueError("Missing required parameters. "
                             "'enrollmentID', 'enrollmentSecret' and 'csr'"
                             " are all required.")

        req = {'certificate_request': csr}
        if self._ca_name != '':
            req.update({
                'caname': self._ca_name
            })
        if profile:
            req.update({
                'profile': profile
            })
        if attr_reqs:
            req.update({
                'attr_reqs': attr_reqs
            })

        res, st = self._send_ca_post(path='enroll',
                                     json=req,
                                     auth=(enrollment_id, enrollment_secret),
                                     verify=self._ca_certs_path)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        if res['success']:
            return base64.b64decode(res['result']['Cert']), \
                base64.b64decode(res['result']['ServerInfo']['CAChain'])
        else:
            raise ValueError("Enrollment failed with errors {0}"
                             .format(res['errors']))

    def register(self, req, registrar):
        authorization = self.generateAuthToken(req, registrar)

        res, st = self._send_ca_post(path="register",
                                     json=req,
                                     headers={'Authorization': authorization},
                                     verify=self._ca_certs_path)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        if res['success']:
            return res['result']['secret']
        else:
            raise ValueError("Registering failed with errors {0}"
                             .format(res['errors']))

    def reenroll(self, req, registrar):
        authorization = self.generateAuthToken(req, registrar)

        res, st = self._send_ca_post(path='reenroll',
                                     json=req,
                                     headers={'Authorization': authorization},
                                     verify=self._ca_certs_path)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        if res['success']:
            return base64.b64decode(res['result']['Cert']), \
                base64.b64decode(res['result']['ServerInfo']['CAChain'])
        else:
            raise ValueError("Reenrollment failed with errors {0}"
                             .format(res['errors']))

    def revoke(self, req, registrar):
        authorization = self.generateAuthToken(req, registrar)

        res, st = self._send_ca_post(path="revoke",
                                     json=req,
                                     headers={'Authorization': authorization},
                                     verify=self._ca_certs_path)

        _logger.debug("Response status {0}".format(st))
        _logger.debug("Raw response json {0}".format(res))

        if res['success']:
            return res['result']['RevokedCerts'], res['result']['CRL']
        else:
            raise ValueError("Revoking failed with errors {0}"
                             .format(res['errors']))

    def generateCRL(self, req, registrar):
        authorization = self.generateAuthToken(req, registrar)
        res, st = self._send_ca_post(path='gencrl',
                                     json=req,
                                     headers={'Authorization': authorization},
                                     verify=self._ca_certs_path)

        _logger.debug('Response status {0}'.format(st))
        _logger.debug('Raw response json {0}'.format(res))

        if res['success']:
            return res['result']['CRL']
        else:
            raise ValueError('generating CRL failed with errors {0}'
                             .format(res['errors']))

    def newIdentityService(self):
        return IdentityService(self)

    def newAffiliationService(self):
        return AffiliationService(self)

    def newCertificateService(self):
        return CertificateService(self)


class CAService(object):
    """This is a ca server delegate."""

    def __init__(self, target=DEFAULT_CA_ENDPOINT,
                 ca_certs_path=None, crypto=ecies(), ca_name=''):
        """ Init CA service.

        :param target: CA server address including protocol, hostname, port
        :param ca_certs_path: Local ca certs path
        :param crypto:  A crypto instance
        :param ca_name: The optional name of the CA, Fabric-ca servers
        support multiple Certificate Authorties from a signle server.
        If omitted or null or an empty string, then the default CA
        is the target of requests
        :return: An instance of CAService
        """
        self._crypto = crypto
        self._ca_client = CAClient(target, ca_certs_path, ca_name=ca_name,
                                   cryptoPrimitives=self._crypto)

    def enroll(self, enrollment_id, enrollment_secret, csr=None, profile='',
               attr_reqs=None):
        """Enroll a registered user in order to receive a signed X509
         certificate

        :param enrollment_id: The registered ID to use for enrollment
        :type enrollment_id: str
        :param enrollment_secret: The secret associated with the
                                     enrollment ID
        :type enrollment_secret: str
        :param profile: The profile name.  Specify the 'tls' profile for a
             TLS certificate; otherwise, an enrollment certificate is issued. (Default value = '')
        :type profile: str
        :param csr: Optional. PEM-encoded PKCS#10 Certificate Signing
             Request. The message sent from client side to Fabric-ca for the
              digital identity certificate. (Default value = None)
        :type csr: str
        :param attr_reqs: An array of AttributeRequest
        :return: PEM-encoded X509 certificate (Default value = None)
        :type attr_reqs: list
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        if attr_reqs:
            if not isinstance(attr_reqs, list):
                raise ValueError("attr_reqs must be an array of"
                                 " AttributeRequest objects")
            else:
                for attr in attr_reqs:
                    if not attr['name']:
                        raise ValueError("attr_reqs object is missing the name"
                                         " of the attribute")

        private_key = None
        if csr:
            _logger.debug("try to enroll with a csr")
        else:
            private_key = self._crypto.generate_private_key()
            csr = self._crypto.generate_csr(private_key, x509.Name(
                [x509.NameAttribute(NameOID.COMMON_NAME,
                                    six.u(enrollment_id))]))

        enrollmentCert, caCertChain = self._ca_client.enroll(
            enrollment_id,
            enrollment_secret,
            csr.public_bytes(Encoding.PEM).decode('utf-8'),
            profile,
            attr_reqs)

        return Enrollment(private_key, enrollmentCert, caCertChain, self)

    def reenroll(self, currentUser, attr_reqs=None):
        """Re-enroll the member in cases such as the existing enrollment
         certificate is about to expire, or it has been compromised

        :param currentUser: The identity of the current user that
             holds the existing enrollment certificate
        :type currentUser: Enrollment
        :param attr_reqs: Optional. An array of AttributeRequest that
             indicate attributes to be included in the certificate
        :return: PEM-encoded X509 certificate (Default value = None)
        :type attr_reqs: list
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """

        if not isinstance(currentUser, Enrollment):
            raise ValueError('"currentUser" is not a valid Enrollment object')

        if attr_reqs:
            if not isinstance(attr_reqs, list):
                raise ValueError("attr_reqs must be an array of"
                                 " AttributeRequest objects")
            else:
                for attr in attr_reqs:
                    if not attr.name:
                        raise ValueError("attr_reqs object is missing the name"
                                         " of the attribute")

        cert = currentUser.cert
        cert = x509.load_pem_x509_certificate(cert, default_backend())

        private_key = self._crypto.generate_private_key()
        csr = self._crypto.generate_csr(private_key, cert.subject)

        req = {'certificate_request': csr.public_bytes(Encoding.PEM).decode(
            'utf-8')}
        if attr_reqs:
            req.update({
                'attr_reqs': attr_reqs
            })

        enrollmentCert, caCertChain = self._ca_client.reenroll(req,
                                                               currentUser)

        return Enrollment(private_key, enrollmentCert, caCertChain, self)

    def register(self, enrollmentID, enrollmentSecret, role, affiliation,
                 maxEnrollments, attrs, registrar):
        """Register a user in order to receive a secret

        :param registrar: The registrar
        :type registrar: Enrollment
        :param enrollmentID: enrollmentID ID which will be used for
             enrollment
        :type enrollmentID: str
        :param enrollmentSecret: enrollmentSecret Optional enrollment secret
             to set for the registered user.
             If not provided, the server will generate one.
             When not including, use a null for this parameter.
        :type enrollmentSecret: str
        :param role: Optional type of role for this user.
                        When not including, use a null for this parameter.
        :type role: str
        :param affiliation: Affiliation with which this user will be
             associated
        :type affiliation: str
        :param maxEnrollments: The maximum number of times the user is
             permitted to enroll
        :type maxEnrollments: number
        :param attrs: Array of key/value attributes to assign to the user
        :return The enrollment secret to use when this user
         enrolls
        :type attrs: dict
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """
        req = {
            "id": enrollmentID,
            "affiliation": affiliation,
            "max_enrollments": maxEnrollments,
        }

        if role:
            req['type'] = role

        if attrs:
            req['attrs'] = attrs

        if isinstance(enrollmentSecret, str) and len(enrollmentSecret):
            req['secret'] = enrollmentSecret

        return self._ca_client.register(req, registrar)

    def revoke(self, enrollmentID, aki, serial, reason, gencrl, registrar):
        """Revoke an existing certificate (enrollment certificate or
         transaction certificate), or revoke all certificates issued to an
          enrollment id. If revoking a particular certificate, then both the
           Authority Key Identifier and serial number are required. If
            revoking by enrollment id, then all future requests to enroll this
             id will be rejected.

        :param registrar: The registrar
        :type registrar: Enrollment
        :param enrollmentID: enrollmentID ID to revoke
        :type enrollmentID: str
        :param aki: Authority Key Identifier string, hex encoded, for the
             specific certificate to revoke
        :type aki: str
        :param serial: Serial number string, hex encoded, for the specific
             certificate to revoke
        :type serial: str
        :param reason: The reason for revocation.
             See https://godoc.org/golang.org/x/crypto/ocsp for valid values
        :type reason: str
        :param gencrl: GenCRL specifies whether to generate a CRL
        :return: The revocation results
        :type gencrl: bool
        :raises RequestException: errors in requests.exceptions
        :raises ValueError: Failed response, json parse error, args missing
        """
        req = {
            "id": enrollmentID,
            "aki": aki,
            "serial": serial,
            "reason": reason,
            "gencrl": gencrl
        }

        if self._ca_client._ca_name != '':
            req.update({
                'caname': self._ca_client._ca_name
            })

        return self._ca_client.revoke(req, registrar)

    def generateCRL(self, revokedBefore, revokedAfter, expireBefore,
                    expireAfter, registrar):
        """Generate CRL

        :param revokedBefore: Include certificates that were revoked before
         this UTC timestamp (in RFC3339 format) in the CRL
        :param revokedAfter: Include certificates that were revoked after
         this UTC timestamp (in RFC3339 format) in the CRL
        :param expireBefore: Include revoked certificates that expire before
         this UTC timestamp (in RFC3339 format) in the CRL
        :param expireAfter: Include revoked certificates that expire after
         this UTC timestamp (in RFC3339 format) in the CRL
        :param registrar: registrar
        :return: The Certificate Revocation List (CRL)
        """
        req = {
            'revokedBefore': revokedBefore,
            'revokedAfter': revokedAfter,
            'expireBefore': expireBefore,
            'expireAfter': expireAfter
        }

        if self._ca_client._ca_name != '':
            req.update({
                'caname': self._ca_client._ca_name
            })

        return self._ca_client.generateCRL(req, registrar)

    def newIdentityService(self):
        return self._ca_client.newIdentityService()

    def newAffiliationService(self):
        return self._ca_client.newAffiliationService()

    def newCertificateService(self):
        return self._ca_client.newCertificateService()


def ca_service(target=DEFAULT_CA_ENDPOINT,
               ca_certs_path=None, crypto=ecies(), ca_name=""):
    """Create ca service

    :param target: url (Default value = DEFAULT_CA_ENDPOINT)
    :param ca_certs_path: certs path (Default value = None)
    :param crypto: crypto (Default value = ecies())
    :param ca_name: CA name
    :return: ca service instance (Default value = "")
    """
    return CAService(target, ca_certs_path, crypto, ca_name)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/ledger/rwset/kvrwset/kv_rwset.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/ledger/rwset/kvrwset/kv_rwset.proto',
  package='kvrwset',
  syntax='proto3',
  serialized_pb=_b('\n.hfc/protos/ledger/rwset/kvrwset/kv_rwset.proto\x12\x07kvrwset\"\x80\x01\n\x07KVRWSet\x12\x1e\n\x05reads\x18\x01 \x03(\x0b\x32\x0f.kvrwset.KVRead\x12\x33\n\x12range_queries_info\x18\x02 \x03(\x0b\x32\x17.kvrwset.RangeQueryInfo\x12 \n\x06writes\x18\x03 \x03(\x0b\x32\x10.kvrwset.KVWrite\"e\n\x0bHashedRWSet\x12)\n\x0chashed_reads\x18\x01 \x03(\x0b\x32\x13.kvrwset.KVReadHash\x12+\n\rhashed_writes\x18\x02 \x03(\x0b\x32\x14.kvrwset.KVWriteHash\"8\n\x06KVRead\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x07version\x18\x02 \x01(\x0b\x32\x10.kvrwset.Version\"8\n\x07KVWrite\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x11\n\tis_delete\x18\x02 \x01(\x08\x12\r\n\x05value\x18\x03 \x01(\x0c\"A\n\nKVReadHash\x12\x10\n\x08key_hash\x18\x01 \x01(\x0c\x12!\n\x07version\x18\x02 \x01(\x0b\x32\x10.kvrwset.Version\"F\n\x0bKVWriteHash\x12\x10\n\x08key_hash\x18\x01 \x01(\x0c\x12\x11\n\tis_delete\x18\x02 \x01(\x08\x12\x12\n\nvalue_hash\x18\x03 \x01(\x0c\",\n\x07Version\x12\x11\n\tblock_num\x18\x01 \x01(\x04\x12\x0e\n\x06tx_num\x18\x02 \x01(\x04\"\xc4\x01\n\x0eRangeQueryInfo\x12\x11\n\tstart_key\x18\x01 \x01(\t\x12\x0f\n\x07\x65nd_key\x18\x02 \x01(\t\x12\x15\n\ritr_exhausted\x18\x03 \x01(\x08\x12(\n\traw_reads\x18\x04 \x01(\x0b\x32\x13.kvrwset.QueryReadsH\x00\x12?\n\x13reads_merkle_hashes\x18\x05 \x01(\x0b\x32 .kvrwset.QueryReadsMerkleSummaryH\x00\x42\x0c\n\nreads_info\"/\n\nQueryReads\x12!\n\x08kv_reads\x18\x01 \x03(\x0b\x32\x0f.kvrwset.KVRead\"Z\n\x17QueryReadsMerkleSummary\x12\x12\n\nmax_degree\x18\x01 \x01(\r\x12\x11\n\tmax_level\x18\x02 \x01(\r\x12\x18\n\x10max_level_hashes\x18\x03 \x03(\x0c\x42o\n2org.hyperledger.fabric.protos.ledger.rwset.kvrwsetZ9github.com/hyperledger/fabric/protos/ledger/rwset/kvrwsetb\x06proto3')
)




_KVRWSET = _descriptor.Descriptor(
  name='KVRWSet',
  full_name='kvrwset.KVRWSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reads', full_name='kvrwset.KVRWSet.reads', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='range_queries_info', full_name='kvrwset.KVRWSet.range_queries_info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='writes', full_name='kvrwset.KVRWSet.writes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=188,
)


_HASHEDRWSET = _descriptor.Descriptor(
  name='HashedRWSet',
  full_name='kvrwset.HashedRWSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hashed_reads', full_name='kvrwset.HashedRWSet.hashed_reads', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hashed_writes', full_name='kvrwset.HashedRWSet.hashed_writes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=291,
)


_KVREAD = _descriptor.Descriptor(
  name='KVRead',
  full_name='kvrwset.KVRead',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='kvrwset.KVRead.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='kvrwset.KVRead.version', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=293,
  serialized_end=349,
)


_KVWRITE = _descriptor.Descriptor(
  name='KVWrite',
  full_name='kvrwset.KVWrite',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='kvrwset.KVWrite.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_delete', full_name='kvrwset.KVWrite.is_delete', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='kvrwset.KVWrite.value', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=351,
  serialized_end=407,
)


_KVREADHASH = _descriptor.Descriptor(
  name='KVReadHash',
  full_name='kvrwset.KVReadHash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_hash', full_name='kvrwset.KVReadHash.key_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='kvrwset.KVReadHash.version', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=409,
  serialized_end=474,
)


_KVWRITEHASH = _descriptor.Descriptor(
  name='KVWriteHash',
  full_name='kvrwset.KVWriteHash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_hash', full_name='kvrwset.KVWriteHash.key_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_delete', full_name='kvrwset.KVWriteHash.is_delete', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_hash', full_name='kvrwset.KVWriteHash.value_hash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=476,
  serialized_end=546,
)


_VERSION = _descriptor.Descriptor(
  name='Version',
  full_name='kvrwset.Version',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_num', full_name='kvrwset.Version.block_num', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tx_num', full_name='kvrwset.Version.tx_num', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=548,
  serialized_end=592,
)


_RANGEQUERYINFO = _descriptor.Descriptor(
  name='RangeQueryInfo',
  full_name='kvrwset.RangeQueryInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_key', full_name='kvrwset.RangeQueryInfo.start_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_key', full_name='kvrwset.RangeQueryInfo.end_key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='itr_exhausted', full_name='kvrwset.RangeQueryInfo.itr_exhausted', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='raw_reads', full_name='kvrwset.RangeQueryInfo.raw_reads', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reads_merkle_hashes', full_name='kvrwset.RangeQueryInfo.reads_merkle_hashes', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='reads_info', full_name='kvrwset.RangeQueryInfo.reads_info',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=595,
  serialized_end=791,
)


_QUERYREADS = _descriptor.Descriptor(
  name='QueryReads',
  full_name='kvrwset.QueryReads',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kv_reads', full_name='kvrwset.QueryReads.kv_reads', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=793,
  serialized_end=840,
)


_QUERYREADSMERKLESUMMARY = _descriptor.Descriptor(
  name='QueryReadsMerkleSummary',
  full_name='kvrwset.QueryReadsMerkleSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_degree', full_name='kvrwset.QueryReadsMerkleSummary.max_degree', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_level', full_name='kvrwset.QueryReadsMerkleSummary.max_level', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_level_hashes', full_name='kvrwset.QueryReadsMerkleSummary.max_level_hashes', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=842,
  serialized_end=932,
)

_KVRWSET.fields_by_name['reads'].message_type = _KVREAD
_KVRWSET.fields_by_name['range_queries_info'].message_type = _RANGEQUERYINFO
_KVRWSET.fields_by_name['writes'].message_type = _KVWRITE
_HASHEDRWSET.fields_by_name['hashed_reads'].message_type = _KVREADHASH
_HASHEDRWSET.fields_by_name['hashed_writes'].message_type = _KVWRITEHASH
_KVREAD.fields_by_name['version'].message_type = _VERSION
_KVREADHASH.fields_by_name['version'].message_type = _VERSION
_RANGEQUERYINFO.fields_by_name['raw_reads'].message_type = _QUERYREADS
_RANGEQUERYINFO.fields_by_name['reads_merkle_hashes'].message_type = _QUERYREADSMERKLESUMMARY
_RANGEQUERYINFO.oneofs_by_name['reads_info'].fields.append(
  _RANGEQUERYINFO.fields_by_name['raw_reads'])
_RANGEQUERYINFO.fields_by_name['raw_reads'].containing_oneof = _RANGEQUERYINFO.oneofs_by_name['reads_info']
_RANGEQUERYINFO.oneofs_by_name['reads_info'].fields.append(
  _RANGEQUERYINFO.fields_by_name['reads_merkle_hashes'])
_RANGEQUERYINFO.fields_by_name['reads_merkle_hashes'].containing_oneof = _RANGEQUERYINFO.oneofs_by_name['reads_info']
_QUERYREADS.fields_by_name['kv_reads'].message_type = _KVREAD
DESCRIPTOR.message_types_by_name['KVRWSet'] = _KVRWSET
DESCRIPTOR.message_types_by_name['HashedRWSet'] = _HASHEDRWSET
DESCRIPTOR.message_types_by_name['KVRead'] = _KVREAD
DESCRIPTOR.message_types_by_name['KVWrite'] = _KVWRITE
DESCRIPTOR.message_types_by_name['KVReadHash'] = _KVREADHASH
DESCRIPTOR.message_types_by_name['KVWriteHash'] = _KVWRITEHASH
DESCRIPTOR.message_types_by_name['Version'] = _VERSION
DESCRIPTOR.message_types_by_name['RangeQueryInfo'] = _RANGEQUERYINFO
DESCRIPTOR.message_types_by_name['QueryReads'] = _QUERYREADS
DESCRIPTOR.message_types_by_name['QueryReadsMerkleSummary'] = _QUERYREADSMERKLESUMMARY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KVRWSet = _reflection.GeneratedProtocolMessageType('KVRWSet', (_message.Message,), dict(
  DESCRIPTOR = _KVRWSET,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.KVRWSet)
  ))
_sym_db.RegisterMessage(KVRWSet)

HashedRWSet = _reflection.GeneratedProtocolMessageType('HashedRWSet', (_message.Message,), dict(
  DESCRIPTOR = _HASHEDRWSET,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.HashedRWSet)
  ))
_sym_db.RegisterMessage(HashedRWSet)

KVRead = _reflection.GeneratedProtocolMessageType('KVRead', (_message.Message,), dict(
  DESCRIPTOR = _KVREAD,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.KVRead)
  ))
_sym_db.RegisterMessage(KVRead)

KVWrite = _reflection.GeneratedProtocolMessageType('KVWrite', (_message.Message,), dict(
  DESCRIPTOR = _KVWRITE,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.KVWrite)
  ))
_sym_db.RegisterMessage(KVWrite)

KVReadHash = _reflection.GeneratedProtocolMessageType('KVReadHash', (_message.Message,), dict(
  DESCRIPTOR = _KVREADHASH,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.KVReadHash)
  ))
_sym_db.RegisterMessage(KVReadHash)

KVWriteHash = _reflection.GeneratedProtocolMessageType('KVWriteHash', (_message.Message,), dict(
  DESCRIPTOR = _KVWRITEHASH,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.KVWriteHash)
  ))
_sym_db.RegisterMessage(KVWriteHash)

Version = _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), dict(
  DESCRIPTOR = _VERSION,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.Version)
  ))
_sym_db.RegisterMessage(Version)

RangeQueryInfo = _reflection.GeneratedProtocolMessageType('RangeQueryInfo', (_message.Message,), dict(
  DESCRIPTOR = _RANGEQUERYINFO,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.RangeQueryInfo)
  ))
_sym_db.RegisterMessage(RangeQueryInfo)

QueryReads = _reflection.GeneratedProtocolMessageType('QueryReads', (_message.Message,), dict(
  DESCRIPTOR = _QUERYREADS,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.QueryReads)
  ))
_sym_db.RegisterMessage(QueryReads)

QueryReadsMerkleSummary = _reflection.GeneratedProtocolMessageType('QueryReadsMerkleSummary', (_message.Message,), dict(
  DESCRIPTOR = _QUERYREADSMERKLESUMMARY,
  __module__ = 'hfc.protos.ledger.rwset.kvrwset.kv_rwset_pb2'
  # @@protoc_insertion_point(class_scope:kvrwset.QueryReadsMerkleSummary)
  ))
_sym_db.RegisterMessage(QueryReadsMerkleSummary)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n2org.hyperledger.fabric.protos.ledger.rwset.kvrwsetZ9github.com/hyperledger/fabric/protos/ledger/rwset/kvrwset'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/configuration.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/configuration.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n#hfc/protos/peer/configuration.proto\x12\x06protos\"7\n\x0b\x41nchorPeers\x12(\n\x0c\x61nchor_peers\x18\x01 \x03(\x0b\x32\x12.protos.AnchorPeer\"(\n\nAnchorPeer\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"!\n\x0b\x41PIResource\x12\x12\n\npolicy_ref\x18\x01 \x01(\t\"n\n\x04\x41\x43Ls\x12$\n\x04\x61\x63ls\x18\x01 \x03(\x0b\x32\x16.protos.ACLs.AclsEntry\x1a@\n\tAclsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.protos.APIResource:\x02\x38\x01\x42O\n\"org.hyperledger.fabric.protos.peerZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
)




_ANCHORPEERS = _descriptor.Descriptor(
  name='AnchorPeers',
  full_name='protos.AnchorPeers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='anchor_peers', full_name='protos.AnchorPeers.anchor_peers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=47,
  serialized_end=102,
)


_ANCHORPEER = _descriptor.Descriptor(
  name='AnchorPeer',
  full_name='protos.AnchorPeer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host', full_name='protos.AnchorPeer.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='protos.AnchorPeer.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=104,
  serialized_end=144,
)


_APIRESOURCE = _descriptor.Descriptor(
  name='APIResource',
  full_name='protos.APIResource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='policy_ref', full_name='protos.APIResource.policy_ref', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=146,
  serialized_end=179,
)


_ACLS_ACLSENTRY = _descriptor.Descriptor(
  name='AclsEntry',
  full_name='protos.ACLs.AclsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='protos.ACLs.AclsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='protos.ACLs.AclsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=227,
  serialized_end=291,
)

_ACLS = _descriptor.Descriptor(
  name='ACLs',
  full_name='protos.ACLs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='acls', full_name='protos.ACLs.acls', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ACLS_ACLSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=291,
)

_ANCHORPEERS.fields_by_name['anchor_peers'].message_type = _ANCHORPEER
_ACLS_ACLSENTRY.fields_by_name['value'].message_type = _APIRESOURCE
_ACLS_ACLSENTRY.containing_type = _ACLS
_ACLS.fields_by_name['acls'].message_type = _ACLS_ACLSENTRY
DESCRIPTOR.message_types_by_name['AnchorPeers'] = _ANCHORPEERS
DESCRIPTOR.message_types_by_name['AnchorPeer'] = _ANCHORPEER
DESCRIPTOR.message_types_by_name['APIResource'] = _APIRESOURCE
DESCRIPTOR.message_types_by_name['ACLs'] = _ACLS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AnchorPeers = _reflection.GeneratedProtocolMessageType('AnchorPeers', (_message.Message,), dict(
  DESCRIPTOR = _ANCHORPEERS,
  __module__ = 'hfc.protos.peer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:protos.AnchorPeers)
  ))
_sym_db.RegisterMessage(AnchorPeers)

AnchorPeer = _reflection.GeneratedProtocolMessageType('AnchorPeer', (_message.Message,), dict(
  DESCRIPTOR = _ANCHORPEER,
  __module__ = 'hfc.protos.peer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:protos.AnchorPeer)
  ))
_sym_db.RegisterMessage(AnchorPeer)

APIResource = _reflection.GeneratedProtocolMessageType('APIResource', (_message.Message,), dict(
  DESCRIPTOR = _APIRESOURCE,
  __module__ = 'hfc.protos.peer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:protos.APIResource)
  ))
_sym_db.RegisterMessage(APIResource)

ACLs = _reflection.GeneratedProtocolMessageType('ACLs', (_message.Message,), dict(

  AclsEntry = _reflection.GeneratedProtocolMessageType('AclsEntry', (_message.Message,), dict(
    DESCRIPTOR = _ACLS_ACLSENTRY,
    __module__ = 'hfc.protos.peer.configuration_pb2'
    # @@protoc_insertion_point(class_scope:protos.ACLs.AclsEntry)
    ))
  ,
  DESCRIPTOR = _ACLS,
  __module__ = 'hfc.protos.peer.configuration_pb2'
  # @@protoc_insertion_point(class_scope:protos.ACLs)
  ))
_sym_db.RegisterMessage(ACLs)
_sym_db.RegisterMessage(ACLs.AclsEntry)


DESCRIPTOR._options = None
_ACLS_ACLSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from hfc.fabric.orderer import Orderer

DEFAULT_ORDERER_ENDPOINT = 'localhost:7050'
CUSTOM_ORDERER_ENDPOINT = 'orderer:7050'


class OrdererTest(unittest.TestCase):

    def setUp(self):
        self.custom_orderer_endpoint = CUSTOM_ORDERER_ENDPOINT
        self.deault_orderer_endpoint = DEFAULT_ORDERER_ENDPOINT

    def test_create_orderer_default_orderer(self):
        orderer = Orderer()
        self.assertEqual(orderer.endpoint, self.deault_orderer_endpoint)

    def test_create_orderer_custom_orderer(self):
        orderer = Orderer(endpoint=self.custom_orderer_endpoint)
        self.assertEqual(orderer.endpoint, self.custom_orderer_endpoint)


if __name__ == '__main__':
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/discovery/protocol.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hfc.protos.gossip import message_pb2 as hfc_dot_protos_dot_gossip_dot_message__pb2
from hfc.protos.msp import msp_config_pb2 as hfc_dot_protos_dot_msp_dot_msp__config__pb2
from hfc.protos.msp import identities_pb2 as hfc_dot_protos_dot_msp_dot_identities__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/discovery/protocol.proto',
  package='discovery',
  syntax='proto3',
  serialized_options=_b('Z.github.com/hyperledger/fabric/protos/discovery'),
  serialized_pb=_b('\n#hfc/protos/discovery/protocol.proto\x12\tdiscovery\x1a\x1fhfc/protos/gossip/message.proto\x1a\x1fhfc/protos/msp/msp_config.proto\x1a\x1fhfc/protos/msp/identities.proto\"3\n\rSignedRequest\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\"Y\n\x07Request\x12+\n\x0e\x61uthentication\x18\x01 \x01(\x0b\x32\x13.discovery.AuthInfo\x12!\n\x07queries\x18\x02 \x03(\x0b\x32\x10.discovery.Query\"3\n\x08Response\x12\'\n\x07results\x18\x01 \x03(\x0b\x32\x16.discovery.QueryResult\"A\n\x08\x41uthInfo\x12\x17\n\x0f\x63lient_identity\x18\x01 \x01(\x0c\x12\x1c\n\x14\x63lient_tls_cert_hash\x18\x02 \x01(\x0c\"\xe8\x01\n\x05Query\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\t\x12.\n\x0c\x63onfig_query\x18\x02 \x01(\x0b\x32\x16.discovery.ConfigQueryH\x00\x12\x34\n\npeer_query\x18\x03 \x01(\x0b\x32\x1e.discovery.PeerMembershipQueryH\x00\x12-\n\x08\x63\x63_query\x18\x04 \x01(\x0b\x32\x19.discovery.ChaincodeQueryH\x00\x12\x30\n\x0blocal_peers\x18\x05 \x01(\x0b\x32\x19.discovery.LocalPeerQueryH\x00\x42\x07\n\x05query\"\xd9\x01\n\x0bQueryResult\x12!\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x10.discovery.ErrorH\x00\x12\x30\n\rconfig_result\x18\x02 \x01(\x0b\x32\x17.discovery.ConfigResultH\x00\x12\x37\n\x0c\x63\x63_query_res\x18\x03 \x01(\x0b\x32\x1f.discovery.ChaincodeQueryResultH\x00\x12\x32\n\x07members\x18\x04 \x01(\x0b\x32\x1f.discovery.PeerMembershipResultH\x00\x42\x08\n\x06result\"\r\n\x0b\x43onfigQuery\"\x82\x02\n\x0c\x43onfigResult\x12/\n\x04msps\x18\x01 \x03(\x0b\x32!.discovery.ConfigResult.MspsEntry\x12\x37\n\x08orderers\x18\x02 \x03(\x0b\x32%.discovery.ConfigResult.OrderersEntry\x1a\x41\n\tMspsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.msp.FabricMSPConfig:\x02\x38\x01\x1a\x45\n\rOrderersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.discovery.Endpoints:\x02\x38\x01\"C\n\x13PeerMembershipQuery\x12,\n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x1c.discovery.ChaincodeInterest\"\xa2\x01\n\x14PeerMembershipResult\x12\x45\n\x0cpeers_by_org\x18\x01 \x03(\x0b\x32/.discovery.PeerMembershipResult.PeersByOrgEntry\x1a\x43\n\x0fPeersByOrgEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.discovery.Peers:\x02\x38\x01\"A\n\x0e\x43haincodeQuery\x12/\n\tinterests\x18\x01 \x03(\x0b\x32\x1c.discovery.ChaincodeInterest\"A\n\x11\x43haincodeInterest\x12,\n\nchaincodes\x18\x01 \x03(\x0b\x32\x18.discovery.ChaincodeCall\"7\n\rChaincodeCall\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x18\n\x10\x63ollection_names\x18\x02 \x03(\t\"I\n\x14\x43haincodeQueryResult\x12\x31\n\x07\x63ontent\x18\x01 \x03(\x0b\x32 .discovery.EndorsementDescriptor\"\x10\n\x0eLocalPeerQuery\"\xf0\x01\n\x15\x45ndorsementDescriptor\x12\x11\n\tchaincode\x18\x01 \x01(\t\x12T\n\x13\x65ndorsers_by_groups\x18\x02 \x03(\x0b\x32\x37.discovery.EndorsementDescriptor.EndorsersByGroupsEntry\x12\"\n\x07layouts\x18\x03 \x03(\x0b\x32\x11.discovery.Layout\x1aJ\n\x16\x45ndorsersByGroupsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.discovery.Peers:\x02\x38\x01\"\x89\x01\n\x06Layout\x12\x45\n\x13quantities_by_group\x18\x01 \x03(\x0b\x32(.discovery.Layout.QuantitiesByGroupEntry\x1a\x38\n\x16QuantitiesByGroupEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\"\'\n\x05Peers\x12\x1e\n\x05peers\x18\x01 \x03(\x0b\x32\x0f.discovery.Peer\"i\n\x04Peer\x12$\n\nstate_info\x18\x01 \x01(\x0b\x32\x10.gossip.Envelope\x12)\n\x0fmembership_info\x18\x02 \x01(\x0b\x32\x10.gossip.Envelope\x12\x10\n\x08identity\x18\x03 \x01(\x0c\"\x18\n\x05\x45rror\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\"2\n\tEndpoints\x12%\n\x08\x65ndpoint\x18\x01 \x03(\x0b\x32\x13.discovery.Endpoint\"&\n\x08\x45ndpoint\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r2H\n\tDiscovery\x12;\n\x08\x44iscover\x12\x18.discovery.SignedRequest\x1a\x13.discovery.Response\"\x00\x42\x30Z.github.com/hyperledger/fabric/protos/discoveryb\x06proto3')
  ,
  dependencies=[hfc_dot_protos_dot_gossip_dot_message__pb2.DESCRIPTOR,hfc_dot_protos_dot_msp_dot_msp__config__pb2.DESCRIPTOR,hfc_dot_protos_dot_msp_dot_identities__pb2.DESCRIPTOR,])




_SIGNEDREQUEST = _descriptor.Descriptor(
  name='SignedRequest',
  full_name='discovery.SignedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='discovery.SignedRequest.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='discovery.SignedRequest.signature', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=200,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='discovery.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authentication', full_name='discovery.Request.authentication', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='queries', full_name='discovery.Request.queries', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=202,
  serialized_end=291,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='discovery.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='discovery.Response.results', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=293,
  serialized_end=344,
)


_AUTHINFO = _descriptor.Descriptor(
  name='AuthInfo',
  full_name='discovery.AuthInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_identity', full_name='discovery.AuthInfo.client_identity', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='client_tls_cert_hash', full_name='discovery.AuthInfo.client_tls_cert_hash', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=346,
  serialized_end=411,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='discovery.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='discovery.Query.channel', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config_query', full_name='discovery.Query.config_query', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peer_query', full_name='discovery.Query.peer_query', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cc_query', full_name='discovery.Query.cc_query', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='local_peers', full_name='discovery.Query.local_peers', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='query', full_name='discovery.Query.query',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=414,
  serialized_end=646,
)


_QUERYRESULT = _descriptor.Descriptor(
  name='QueryResult',
  full_name='discovery.QueryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='discovery.QueryResult.error', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config_result', full_name='discovery.QueryResult.config_result', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cc_query_res', full_name='discovery.QueryResult.cc_query_res', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='members', full_name='discovery.QueryResult.members', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='result', full_name='discovery.QueryResult.result',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=649,
  serialized_end=866,
)


_CONFIGQUERY = _descriptor.Descriptor(
  name='ConfigQuery',
  full_name='discovery.ConfigQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=881,
)


_CONFIGRESULT_MSPSENTRY = _descriptor.Descriptor(
  name='MspsEntry',
  full_name='discovery.ConfigResult.MspsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='discovery.ConfigResult.MspsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='discovery.ConfigResult.MspsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1006,
  serialized_end=1071,
)

_CONFIGRESULT_ORDERERSENTRY = _descriptor.Descriptor(
  name='OrderersEntry',
  full_name='discovery.ConfigResult.OrderersEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='discovery.ConfigResult.OrderersEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='discovery.ConfigResult.OrderersEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1073,
  serialized_end=1142,
)

_CONFIGRESULT = _descriptor.Descriptor(
  name='ConfigResult',
  full_name='discovery.ConfigResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='msps', full_name='discovery.ConfigResult.msps', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='orderers', full_name='discovery.ConfigResult.orderers', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONFIGRESULT_MSPSENTRY, _CONFIGRESULT_ORDERERSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=884,
  serialized_end=1142,
)


_PEERMEMBERSHIPQUERY = _descriptor.Descriptor(
  name='PeerMembershipQuery',
  full_name='discovery.PeerMembershipQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='discovery.PeerMembershipQuery.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1144,
  serialized_end=1211,
)


_PEERMEMBERSHIPRESULT_PEERSBYORGENTRY = _descriptor.Descriptor(
  name='PeersByOrgEntry',
  full_name='discovery.PeerMembershipResult.PeersByOrgEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='discovery.PeerMembershipResult.PeersByOrgEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='discovery.PeerMembershipResult.PeersByOrgEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1309,
  serialized_end=1376,
)

_PEERMEMBERSHIPRESULT = _descriptor.Descriptor(
  name='PeerMembershipResult',
  full_name='discovery.PeerMembershipResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='peers_by_org', full_name='discovery.PeerMembershipResult.peers_by_org', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PEERMEMBERSHIPRESULT_PEERSBYORGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1214,
  serialized_end=1376,
)


_CHAINCODEQUERY = _descriptor.Descriptor(
  name='ChaincodeQuery',
  full_name='discovery.ChaincodeQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='interests', full_name='discovery.ChaincodeQuery.interests', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1378,
  serialized_end=1443,
)


_CHAINCODEINTEREST = _descriptor.Descriptor(
  name='ChaincodeInterest',
  full_name='discovery.ChaincodeInterest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincodes', full_name='discovery.ChaincodeInterest.chaincodes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1445,
  serialized_end=1510,
)


_CHAINCODECALL = _descriptor.Descriptor(
  name='ChaincodeCall',
  full_name='discovery.ChaincodeCall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='discovery.ChaincodeCall.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='collection_names', full_name='discovery.ChaincodeCall.collection_names', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1512,
  serialized_end=1567,
)


_CHAINCODEQUERYRESULT = _descriptor.Descriptor(
  name='ChaincodeQueryResult',
  full_name='discovery.ChaincodeQueryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='discovery.ChaincodeQueryResult.content', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1569,
  serialized_end=1642,
)


_LOCALPEERQUERY = _descriptor.Descriptor(
  name='LocalPeerQuery',
  full_name='discovery.LocalPeerQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1644,
  serialized_end=1660,
)


_ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY = _descriptor.Descriptor(
  name='EndorsersByGroupsEntry',
  full_name='discovery.EndorsementDescriptor.EndorsersByGroupsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='discovery.EndorsementDescriptor.EndorsersByGroupsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='discovery.EndorsementDescriptor.EndorsersByGroupsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1829,
  serialized_end=1903,
)

_ENDORSEMENTDESCRIPTOR = _descriptor.Descriptor(
  name='EndorsementDescriptor',
  full_name='discovery.EndorsementDescriptor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chaincode', full_name='discovery.EndorsementDescriptor.chaincode', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endorsers_by_groups', full_name='discovery.EndorsementDescriptor.endorsers_by_groups', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='layouts', full_name='discovery.EndorsementDescriptor.layouts', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1663,
  serialized_end=1903,
)


_LAYOUT_QUANTITIESBYGROUPENTRY = _descriptor.Descriptor(
  name='QuantitiesByGroupEntry',
  full_name='discovery.Layout.QuantitiesByGroupEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='discovery.Layout.QuantitiesByGroupEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='discovery.Layout.QuantitiesByGroupEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1987,
  serialized_end=2043,
)

_LAYOUT = _descriptor.Descriptor(
  name='Layout',
  full_name='discovery.Layout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='quantities_by_group', full_name='discovery.Layout.quantities_by_group', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_LAYOUT_QUANTITIESBYGROUPENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1906,
  serialized_end=2043,
)


_PEERS = _descriptor.Descriptor(
  name='Peers',
  full_name='discovery.Peers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='peers', full_name='discovery.Peers.peers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2045,
  serialized_end=2084,
)


_PEER = _descriptor.Descriptor(
  name='Peer',
  full_name='discovery.Peer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state_info', full_name='discovery.Peer.state_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='membership_info', full_name='discovery.Peer.membership_info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='identity', full_name='discovery.Peer.identity', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2086,
  serialized_end=2191,
)


_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='discovery.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='discovery.Error.content', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2193,
  serialized_end=2217,
)


_ENDPOINTS = _descriptor.Descriptor(
  name='Endpoints',
  full_name='discovery.Endpoints',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='discovery.Endpoints.endpoint', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2219,
  serialized_end=2269,
)


_ENDPOINT = _descriptor.Descriptor(
  name='Endpoint',
  full_name='discovery.Endpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host', full_name='discovery.Endpoint.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='discovery.Endpoint.port', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2271,
  serialized_end=2309,
)

_REQUEST.fields_by_name['authentication'].message_type = _AUTHINFO
_REQUEST.fields_by_name['queries'].message_type = _QUERY
_RESPONSE.fields_by_name['results'].message_type = _QUERYRESULT
_QUERY.fields_by_name['config_query'].message_type = _CONFIGQUERY
_QUERY.fields_by_name['peer_query'].message_type = _PEERMEMBERSHIPQUERY
_QUERY.fields_by_name['cc_query'].message_type = _CHAINCODEQUERY
_QUERY.fields_by_name['local_peers'].message_type = _LOCALPEERQUERY
_QUERY.oneofs_by_name['query'].fields.append(
  _QUERY.fields_by_name['config_query'])
_QUERY.fields_by_name['config_query'].containing_oneof = _QUERY.oneofs_by_name['query']
_QUERY.oneofs_by_name['query'].fields.append(
  _QUERY.fields_by_name['peer_query'])
_QUERY.fields_by_name['peer_query'].containing_oneof = _QUERY.oneofs_by_name['query']
_QUERY.oneofs_by_name['query'].fields.append(
  _QUERY.fields_by_name['cc_query'])
_QUERY.fields_by_name['cc_query'].containing_oneof = _QUERY.oneofs_by_name['query']
_QUERY.oneofs_by_name['query'].fields.append(
  _QUERY.fields_by_name['local_peers'])
_QUERY.fields_by_name['local_peers'].containing_oneof = _QUERY.oneofs_by_name['query']
_QUERYRESULT.fields_by_name['error'].message_type = _ERROR
_QUERYRESULT.fields_by_name['config_result'].message_type = _CONFIGRESULT
_QUERYRESULT.fields_by_name['cc_query_res'].message_type = _CHAINCODEQUERYRESULT
_QUERYRESULT.fields_by_name['members'].message_type = _PEERMEMBERSHIPRESULT
_QUERYRESULT.oneofs_by_name['result'].fields.append(
  _QUERYRESULT.fields_by_name['error'])
_QUERYRESULT.fields_by_name['error'].containing_oneof = _QUERYRESULT.oneofs_by_name['result']
_QUERYRESULT.oneofs_by_name['result'].fields.append(
  _QUERYRESULT.fields_by_name['config_result'])
_QUERYRESULT.fields_by_name['config_result'].containing_oneof = _QUERYRESULT.oneofs_by_name['result']
_QUERYRESULT.oneofs_by_name['result'].fields.append(
  _QUERYRESULT.fields_by_name['cc_query_res'])
_QUERYRESULT.fields_by_name['cc_query_res'].containing_oneof = _QUERYRESULT.oneofs_by_name['result']
_QUERYRESULT.oneofs_by_name['result'].fields.append(
  _QUERYRESULT.fields_by_name['members'])
_QUERYRESULT.fields_by_name['members'].containing_oneof = _QUERYRESULT.oneofs_by_name['result']
_CONFIGRESULT_MSPSENTRY.fields_by_name['value'].message_type = hfc_dot_protos_dot_msp_dot_msp__config__pb2._FABRICMSPCONFIG
_CONFIGRESULT_MSPSENTRY.containing_type = _CONFIGRESULT
_CONFIGRESULT_ORDERERSENTRY.fields_by_name['value'].message_type = _ENDPOINTS
_CONFIGRESULT_ORDERERSENTRY.containing_type = _CONFIGRESULT
_CONFIGRESULT.fields_by_name['msps'].message_type = _CONFIGRESULT_MSPSENTRY
_CONFIGRESULT.fields_by_name['orderers'].message_type = _CONFIGRESULT_ORDERERSENTRY
_PEERMEMBERSHIPQUERY.fields_by_name['filter'].message_type = _CHAINCODEINTEREST
_PEERMEMBERSHIPRESULT_PEERSBYORGENTRY.fields_by_name['value'].message_type = _PEERS
_PEERMEMBERSHIPRESULT_PEERSBYORGENTRY.containing_type = _PEERMEMBERSHIPRESULT
_PEERMEMBERSHIPRESULT.fields_by_name['peers_by_org'].message_type = _PEERMEMBERSHIPRESULT_PEERSBYORGENTRY
_CHAINCODEQUERY.fields_by_name['interests'].message_type = _CHAINCODEINTEREST
_CHAINCODEINTEREST.fields_by_name['chaincodes'].message_type = _CHAINCODECALL
_CHAINCODEQUERYRESULT.fields_by_name['content'].message_type = _ENDORSEMENTDESCRIPTOR
_ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY.fields_by_name['value'].message_type = _PEERS
_ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY.containing_type = _ENDORSEMENTDESCRIPTOR
_ENDORSEMENTDESCRIPTOR.fields_by_name['endorsers_by_groups'].message_type = _ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY
_ENDORSEMENTDESCRIPTOR.fields_by_name['layouts'].message_type = _LAYOUT
_LAYOUT_QUANTITIESBYGROUPENTRY.containing_type = _LAYOUT
_LAYOUT.fields_by_name['quantities_by_group'].message_type = _LAYOUT_QUANTITIESBYGROUPENTRY
_PEERS.fields_by_name['peers'].message_type = _PEER
_PEER.fields_by_name['state_info'].message_type = hfc_dot_protos_dot_gossip_dot_message__pb2._ENVELOPE
_PEER.fields_by_name['membership_info'].message_type = hfc_dot_protos_dot_gossip_dot_message__pb2._ENVELOPE
_ENDPOINTS.fields_by_name['endpoint'].message_type = _ENDPOINT
DESCRIPTOR.message_types_by_name['SignedRequest'] = _SIGNEDREQUEST
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['AuthInfo'] = _AUTHINFO
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['QueryResult'] = _QUERYRESULT
DESCRIPTOR.message_types_by_name['ConfigQuery'] = _CONFIGQUERY
DESCRIPTOR.message_types_by_name['ConfigResult'] = _CONFIGRESULT
DESCRIPTOR.message_types_by_name['PeerMembershipQuery'] = _PEERMEMBERSHIPQUERY
DESCRIPTOR.message_types_by_name['PeerMembershipResult'] = _PEERMEMBERSHIPRESULT
DESCRIPTOR.message_types_by_name['ChaincodeQuery'] = _CHAINCODEQUERY
DESCRIPTOR.message_types_by_name['ChaincodeInterest'] = _CHAINCODEINTEREST
DESCRIPTOR.message_types_by_name['ChaincodeCall'] = _CHAINCODECALL
DESCRIPTOR.message_types_by_name['ChaincodeQueryResult'] = _CHAINCODEQUERYRESULT
DESCRIPTOR.message_types_by_name['LocalPeerQuery'] = _LOCALPEERQUERY
DESCRIPTOR.message_types_by_name['EndorsementDescriptor'] = _ENDORSEMENTDESCRIPTOR
DESCRIPTOR.message_types_by_name['Layout'] = _LAYOUT
DESCRIPTOR.message_types_by_name['Peers'] = _PEERS
DESCRIPTOR.message_types_by_name['Peer'] = _PEER
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
DESCRIPTOR.message_types_by_name['Endpoints'] = _ENDPOINTS
DESCRIPTOR.message_types_by_name['Endpoint'] = _ENDPOINT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SignedRequest = _reflection.GeneratedProtocolMessageType('SignedRequest', (_message.Message,), dict(
  DESCRIPTOR = _SIGNEDREQUEST,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.SignedRequest)
  ))
_sym_db.RegisterMessage(SignedRequest)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Response)
  ))
_sym_db.RegisterMessage(Response)

AuthInfo = _reflection.GeneratedProtocolMessageType('AuthInfo', (_message.Message,), dict(
  DESCRIPTOR = _AUTHINFO,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.AuthInfo)
  ))
_sym_db.RegisterMessage(AuthInfo)

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,), dict(
  DESCRIPTOR = _QUERY,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Query)
  ))
_sym_db.RegisterMessage(Query)

QueryResult = _reflection.GeneratedProtocolMessageType('QueryResult', (_message.Message,), dict(
  DESCRIPTOR = _QUERYRESULT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.QueryResult)
  ))
_sym_db.RegisterMessage(QueryResult)

ConfigQuery = _reflection.GeneratedProtocolMessageType('ConfigQuery', (_message.Message,), dict(
  DESCRIPTOR = _CONFIGQUERY,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ConfigQuery)
  ))
_sym_db.RegisterMessage(ConfigQuery)

ConfigResult = _reflection.GeneratedProtocolMessageType('ConfigResult', (_message.Message,), dict(

  MspsEntry = _reflection.GeneratedProtocolMessageType('MspsEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGRESULT_MSPSENTRY,
    __module__ = 'hfc.protos.discovery.protocol_pb2'
    # @@protoc_insertion_point(class_scope:discovery.ConfigResult.MspsEntry)
    ))
  ,

  OrderersEntry = _reflection.GeneratedProtocolMessageType('OrderersEntry', (_message.Message,), dict(
    DESCRIPTOR = _CONFIGRESULT_ORDERERSENTRY,
    __module__ = 'hfc.protos.discovery.protocol_pb2'
    # @@protoc_insertion_point(class_scope:discovery.ConfigResult.OrderersEntry)
    ))
  ,
  DESCRIPTOR = _CONFIGRESULT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ConfigResult)
  ))
_sym_db.RegisterMessage(ConfigResult)
_sym_db.RegisterMessage(ConfigResult.MspsEntry)
_sym_db.RegisterMessage(ConfigResult.OrderersEntry)

PeerMembershipQuery = _reflection.GeneratedProtocolMessageType('PeerMembershipQuery', (_message.Message,), dict(
  DESCRIPTOR = _PEERMEMBERSHIPQUERY,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.PeerMembershipQuery)
  ))
_sym_db.RegisterMessage(PeerMembershipQuery)

PeerMembershipResult = _reflection.GeneratedProtocolMessageType('PeerMembershipResult', (_message.Message,), dict(

  PeersByOrgEntry = _reflection.GeneratedProtocolMessageType('PeersByOrgEntry', (_message.Message,), dict(
    DESCRIPTOR = _PEERMEMBERSHIPRESULT_PEERSBYORGENTRY,
    __module__ = 'hfc.protos.discovery.protocol_pb2'
    # @@protoc_insertion_point(class_scope:discovery.PeerMembershipResult.PeersByOrgEntry)
    ))
  ,
  DESCRIPTOR = _PEERMEMBERSHIPRESULT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.PeerMembershipResult)
  ))
_sym_db.RegisterMessage(PeerMembershipResult)
_sym_db.RegisterMessage(PeerMembershipResult.PeersByOrgEntry)

ChaincodeQuery = _reflection.GeneratedProtocolMessageType('ChaincodeQuery', (_message.Message,), dict(
  DESCRIPTOR = _CHAINCODEQUERY,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ChaincodeQuery)
  ))
_sym_db.RegisterMessage(ChaincodeQuery)

ChaincodeInterest = _reflection.GeneratedProtocolMessageType('ChaincodeInterest', (_message.Message,), dict(
  DESCRIPTOR = _CHAINCODEINTEREST,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ChaincodeInterest)
  ))
_sym_db.RegisterMessage(ChaincodeInterest)

ChaincodeCall = _reflection.GeneratedProtocolMessageType('ChaincodeCall', (_message.Message,), dict(
  DESCRIPTOR = _CHAINCODECALL,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ChaincodeCall)
  ))
_sym_db.RegisterMessage(ChaincodeCall)

ChaincodeQueryResult = _reflection.GeneratedProtocolMessageType('ChaincodeQueryResult', (_message.Message,), dict(
  DESCRIPTOR = _CHAINCODEQUERYRESULT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.ChaincodeQueryResult)
  ))
_sym_db.RegisterMessage(ChaincodeQueryResult)

LocalPeerQuery = _reflection.GeneratedProtocolMessageType('LocalPeerQuery', (_message.Message,), dict(
  DESCRIPTOR = _LOCALPEERQUERY,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.LocalPeerQuery)
  ))
_sym_db.RegisterMessage(LocalPeerQuery)

EndorsementDescriptor = _reflection.GeneratedProtocolMessageType('EndorsementDescriptor', (_message.Message,), dict(

  EndorsersByGroupsEntry = _reflection.GeneratedProtocolMessageType('EndorsersByGroupsEntry', (_message.Message,), dict(
    DESCRIPTOR = _ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY,
    __module__ = 'hfc.protos.discovery.protocol_pb2'
    # @@protoc_insertion_point(class_scope:discovery.EndorsementDescriptor.EndorsersByGroupsEntry)
    ))
  ,
  DESCRIPTOR = _ENDORSEMENTDESCRIPTOR,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.EndorsementDescriptor)
  ))
_sym_db.RegisterMessage(EndorsementDescriptor)
_sym_db.RegisterMessage(EndorsementDescriptor.EndorsersByGroupsEntry)

Layout = _reflection.GeneratedProtocolMessageType('Layout', (_message.Message,), dict(

  QuantitiesByGroupEntry = _reflection.GeneratedProtocolMessageType('QuantitiesByGroupEntry', (_message.Message,), dict(
    DESCRIPTOR = _LAYOUT_QUANTITIESBYGROUPENTRY,
    __module__ = 'hfc.protos.discovery.protocol_pb2'
    # @@protoc_insertion_point(class_scope:discovery.Layout.QuantitiesByGroupEntry)
    ))
  ,
  DESCRIPTOR = _LAYOUT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Layout)
  ))
_sym_db.RegisterMessage(Layout)
_sym_db.RegisterMessage(Layout.QuantitiesByGroupEntry)

Peers = _reflection.GeneratedProtocolMessageType('Peers', (_message.Message,), dict(
  DESCRIPTOR = _PEERS,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Peers)
  ))
_sym_db.RegisterMessage(Peers)

Peer = _reflection.GeneratedProtocolMessageType('Peer', (_message.Message,), dict(
  DESCRIPTOR = _PEER,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Peer)
  ))
_sym_db.RegisterMessage(Peer)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
  DESCRIPTOR = _ERROR,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Error)
  ))
_sym_db.RegisterMessage(Error)

Endpoints = _reflection.GeneratedProtocolMessageType('Endpoints', (_message.Message,), dict(
  DESCRIPTOR = _ENDPOINTS,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Endpoints)
  ))
_sym_db.RegisterMessage(Endpoints)

Endpoint = _reflection.GeneratedProtocolMessageType('Endpoint', (_message.Message,), dict(
  DESCRIPTOR = _ENDPOINT,
  __module__ = 'hfc.protos.discovery.protocol_pb2'
  # @@protoc_insertion_point(class_scope:discovery.Endpoint)
  ))
_sym_db.RegisterMessage(Endpoint)


DESCRIPTOR._options = None
_CONFIGRESULT_MSPSENTRY._options = None
_CONFIGRESULT_ORDERERSENTRY._options = None
_PEERMEMBERSHIPRESULT_PEERSBYORGENTRY._options = None
_ENDORSEMENTDESCRIPTOR_ENDORSERSBYGROUPSENTRY._options = None
_LAYOUT_QUANTITIESBYGROUPENTRY._options = None

_DISCOVERY = _descriptor.ServiceDescriptor(
  name='Discovery',
  full_name='discovery.Discovery',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2311,
  serialized_end=2383,
  methods=[
  _descriptor.MethodDescriptor(
    name='Discover',
    full_name='discovery.Discovery.Discover',
    index=0,
    containing_service=None,
    input_type=_SIGNEDREQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DISCOVERY)

DESCRIPTOR.services_by_name['Discovery'] = _DISCOVERY

# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hfc/protos/peer/proposal_response.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hfc/protos/peer/proposal_response.proto',
  package='protos',
  syntax='proto3',
  serialized_options=_b('\n\"org.hyperledger.fabric.protos.peerB\027ProposalResponsePackageZ)github.com/hyperledger/fabric/protos/peer'),
  serialized_pb=_b('\n\'hfc/protos/peer/proposal_response.proto\x12\x06protos\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb1\x01\n\x10ProposalResponse\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\"\n\x08response\x18\x04 \x01(\x0b\x32\x10.protos.Response\x12\x0f\n\x07payload\x18\x05 \x01(\x0c\x12(\n\x0b\x65ndorsement\x18\x06 \x01(\x0b\x32\x13.protos.Endorsement\"<\n\x08Response\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\"C\n\x17ProposalResponsePayload\x12\x15\n\rproposal_hash\x18\x01 \x01(\x0c\x12\x11\n\textension\x18\x02 \x01(\x0c\"2\n\x0b\x45ndorsement\x12\x10\n\x08\x65ndorser\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x42h\n\"org.hyperledger.fabric.protos.peerB\x17ProposalResponsePackageZ)github.com/hyperledger/fabric/protos/peerb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_PROPOSALRESPONSE = _descriptor.Descriptor(
  name='ProposalResponse',
  full_name='protos.ProposalResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='protos.ProposalResponse.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='protos.ProposalResponse.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response', full_name='protos.ProposalResponse.response', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='protos.ProposalResponse.payload', index=3,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='endorsement', full_name='protos.ProposalResponse.endorsement', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=262,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='protos.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='protos.Response.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='protos.Response.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='protos.Response.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=264,
  serialized_end=324,
)


_PROPOSALRESPONSEPAYLOAD = _descriptor.Descriptor(
  name='ProposalResponsePayload',
  full_name='protos.ProposalResponsePayload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='proposal_hash', full_name='protos.ProposalResponsePayload.proposal_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extension', full_name='protos.ProposalResponsePayload.extension', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=326,
  serialized_end=393,
)


_ENDORSEMENT = _descriptor.Descriptor(
  name='Endorsement',
  full_name='protos.Endorsement',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='endorser', full_name='protos.Endorsement.endorser', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='protos.Endorsement.signature', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=395,
  serialized_end=445,
)

_PROPOSALRESPONSE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROPOSALRESPONSE.fields_by_name['response'].message_type = _RESPONSE
_PROPOSALRESPONSE.fields_by_name['endorsement'].message_type = _ENDORSEMENT
DESCRIPTOR.message_types_by_name['ProposalResponse'] = _PROPOSALRESPONSE
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['ProposalResponsePayload'] = _PROPOSALRESPONSEPAYLOAD
DESCRIPTOR.message_types_by_name['Endorsement'] = _ENDORSEMENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProposalResponse = _reflection.GeneratedProtocolMessageType('ProposalResponse', (_message.Message,), dict(
  DESCRIPTOR = _PROPOSALRESPONSE,
  __module__ = 'hfc.protos.peer.proposal_response_pb2'
  # @@protoc_insertion_point(class_scope:protos.ProposalResponse)
  ))
_sym_db.RegisterMessage(ProposalResponse)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'hfc.protos.peer.proposal_response_pb2'
  # @@protoc_insertion_point(class_scope:protos.Response)
  ))
_sym_db.RegisterMessage(Response)

ProposalResponsePayload = _reflection.GeneratedProtocolMessageType('ProposalResponsePayload', (_message.Message,), dict(
  DESCRIPTOR = _PROPOSALRESPONSEPAYLOAD,
  __module__ = 'hfc.protos.peer.proposal_response_pb2'
  # @@protoc_insertion_point(class_scope:protos.ProposalResponsePayload)
  ))
_sym_db.RegisterMessage(ProposalResponsePayload)

Endorsement = _reflection.GeneratedProtocolMessageType('Endorsement', (_message.Message,), dict(
  DESCRIPTOR = _ENDORSEMENT,
  __module__ = 'hfc.protos.peer.proposal_response_pb2'
  # @@protoc_insertion_point(class_scope:protos.Endorsement)
  ))
_sym_db.RegisterMessage(Endorsement)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from hfc.protos.common import common_pb2 as hfc_dot_protos_dot_common_dot_common__pb2
from hfc.protos.peer import admin_pb2 as hfc_dot_protos_dot_peer_dot_admin__pb2


class AdminStub(object):
  """Interface exported by the server.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetStatus = channel.unary_unary(
        '/protos.Admin/GetStatus',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.ServerStatus.FromString,
        )
    self.StartServer = channel.unary_unary(
        '/protos.Admin/StartServer',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.ServerStatus.FromString,
        )
    self.GetModuleLogLevel = channel.unary_unary(
        '/protos.Admin/GetModuleLogLevel',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogLevelResponse.FromString,
        )
    self.SetModuleLogLevel = channel.unary_unary(
        '/protos.Admin/SetModuleLogLevel',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogLevelResponse.FromString,
        )
    self.RevertLogLevels = channel.unary_unary(
        '/protos.Admin/RevertLogLevels',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
    self.GetLogSpec = channel.unary_unary(
        '/protos.Admin/GetLogSpec',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogSpecResponse.FromString,
        )
    self.SetLogSpec = channel.unary_unary(
        '/protos.Admin/SetLogSpec',
        request_serializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.SerializeToString,
        response_deserializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogSpecResponse.FromString,
        )


class AdminServicer(object):
  """Interface exported by the server.
  """

  def GetStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StartServer(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetModuleLogLevel(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SetModuleLogLevel(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RevertLogLevels(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetLogSpec(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SetLogSpec(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AdminServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetStatus': grpc.unary_unary_rpc_method_handler(
          servicer.GetStatus,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.ServerStatus.SerializeToString,
      ),
      'StartServer': grpc.unary_unary_rpc_method_handler(
          servicer.StartServer,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.ServerStatus.SerializeToString,
      ),
      'GetModuleLogLevel': grpc.unary_unary_rpc_method_handler(
          servicer.GetModuleLogLevel,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogLevelResponse.SerializeToString,
      ),
      'SetModuleLogLevel': grpc.unary_unary_rpc_method_handler(
          servicer.SetModuleLogLevel,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogLevelResponse.SerializeToString,
      ),
      'RevertLogLevels': grpc.unary_unary_rpc_method_handler(
          servicer.RevertLogLevels,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
      ),
      'GetLogSpec': grpc.unary_unary_rpc_method_handler(
          servicer.GetLogSpec,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogSpecResponse.SerializeToString,
      ),
      'SetLogSpec': grpc.unary_unary_rpc_method_handler(
          servicer.SetLogSpec,
          request_deserializer=hfc_dot_protos_dot_common_dot_common__pb2.Envelope.FromString,
          response_serializer=hfc_dot_protos_dot_peer_dot_admin__pb2.LogSpecResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'protos.Admin', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
